import math
import numpy as np

import gym
from gym import spaces
from gym.utils import seeding
import time

WIDTH = 400
HEIGHT = 400
radar_count = 6
stxaxis = 20
styaxis = 20
endxaxis = 380
endyaxis = 380
radxaxis = [340,280,220,160,100,40]
radyaxis = [40,100,160,220,280,340]
radrange = [50,40,50,40,50,40]
start = time.time()
v=0.6

class OptimalPathEnv(gym.Env):
    metadata = {
        'render.modes': ['human', 'rgb_array'],
        'video.frames_per_second': 30
    }
    hardcore = False

    def __init__(self):
        self.u = 0.0
        self.min_action = 0
        self.max_action = 0.942
        self.minrcs1 = 0.0
        self.maxrcs1 = 1000000
        self.minsnr1 = 0.0
        self.maxsnr1 = 10000000000000
        self.minrcs2 = 0.0
        self.maxrcs2 = 1000000
        self.minsnr2 = 0.0
        self.maxsnr2 = 10000000000000
        self.minrcs3 = 0.0
        self.maxrcs3 = 1000000
        self.minsnr3 = 0.0
        self.maxsnr3 = 10000000000000
        self.minrcs4 = 0.0
        self.maxrcs4 = 1000000
        self.minsnr4 = 0.0
        self.maxsnr4 = 10000000000000
        self.minrcs5 = 0.0
        self.maxrcs5 = 1000000
        self.minsnr5 = 0.0
        self.maxsnr5 = 10000000000000
        self.minrcs6 = 0.0
        self.maxrcs6 = 1000000
        self.minsnr6 = 0.0
        self.maxsnr6 = 10000000000000
        self.min_positionx = stxaxis
        self.max_positionx = endxaxis
        self.min_positiony = styaxis
        self.max_positiony = endyaxis
        self.target_positionx = endxaxis
        self.target_positiony = endyaxis
        self.t = 1
        self.angle = 360
        self.radxaxis1 = radxaxis[0]
        self.radyaxis1 = radyaxis[0]
        self.radrange1 = radrange[0]
        self.radxaxis2 = radxaxis[1]
        self.radyaxis2 = radyaxis[1]
        self.radrange2 = radrange[1]
        self.radxaxis3 = radxaxis[2]
        self.radyaxis3 = radyaxis[2]
        self.radrange3 = radrange[2]
        self.radxaxis4 = radxaxis[3]
        self.radyaxis4 = radyaxis[3]
        self.radrange4 = radrange[3]
        self.radxaxis5 = radxaxis[4]
        self.radyaxis5 = radyaxis[4]
        self.radrange5 = radrange[4]
        self.radxaxis6 = radxaxis[5]
        self.radyaxis6 = radyaxis[5]
        self.radrange6 = radrange[5]
        
        self.minradx1 = self.radxaxis1 - self.radrange1
        self.minrady1 = self.radyaxis1 - self.radrange1
        self.maxradx1 = self.radxaxis1 + self.radrange1
        self.maxrady1 = self.radyaxis1 + self.radrange1
        
        self.minradx2 = self.radxaxis2 - self.radrange2
        self.minrady2 = self.radyaxis2 - self.radrange2
        self.maxradx2 = self.radxaxis2 + self.radrange2
        self.maxrady2 = self.radyaxis2 + self.radrange2
        
        self.minradx3 = self.radxaxis3 - self.radrange3
        self.minrady3 = self.radyaxis3 - self.radrange3
        self.maxradx3 = self.radxaxis3 + self.radrange3
        self.maxrady3 = self.radyaxis3 + self.radrange3
        
        self.minradx4 = self.radxaxis4 - self.radrange4
        self.minrady4 = self.radyaxis4 - self.radrange4
        self.maxradx4 = self.radxaxis4 + self.radrange4
        self.maxrady4 = self.radyaxis4 + self.radrange4
        
        self.minradx5 = self.radxaxis5 - self.radrange5
        self.minrady5 = self.radyaxis5 - self.radrange5
        self.maxradx5 = self.radxaxis5 + self.radrange5
        self.maxrady5 = self.radyaxis5 + self.radrange5
        
        self.minradx6 = self.radxaxis6 - self.radrange6
        self.minrady6 = self.radyaxis6 - self.radrange6
        self.maxradx6 = self.radxaxis6 + self.radrange6
        self.maxrady6 = self.radyaxis6 + self.radrange6

        self.low_state = np.array([self.min_positionx, self.min_positiony, self.minrcs1, self.minsnr1, self.minrcs2, self.minsnr2, self.minrcs3, self.minsnr3, self.minrcs4, self.minsnr4, self.minrcs5, self.minsnr5, self.minrcs6, self.minsnr6])
        self.high_state = np.array([self.max_positionx, self.max_positiony, self.maxrcs1, self.maxsnr1, self.maxrcs2, self.maxsnr2, self.maxrcs3, self.maxsnr3, self.maxrcs4, self.maxsnr4, self.maxrcs5, self.maxsnr5, self.maxrcs6, self.maxsnr6])

        self.viewer = None

        self.action_space = spaces.Box(low=self.min_action, high=self.max_action,
                                       shape=(1,), dtype=np.float32)
        self.observation_space = spaces.Box(low=self.low_state, high=self.high_state,
                                            dtype=np.float32)

        self.seed()
        self.reset()

    def seed(self, seed=None):
        self.np_random, seed = seeding.np_random(seed)
        return [seed]

    def step(self, action):

        positionx = self.state[0]
        positiony = self.state[1]
        rcs1 = self.state[2]
        snr1 = self.state[3]
        rcs2 = self.state[4]
        snr2 = self.state[5]
        rcs3 = self.state[6]
        snr3 = self.state[7]
        rcs4 = self.state[8]
        snr4 = self.state[9]
        rcs5 = self.state[10]
        snr5 = self.state[11]
        rcs6 = self.state[12]
        snr6 = self.state[13]
        self.u = min(max(action[0], self.min_action), self.max_action)
        if self.u > -0.2355 and self.u < 0.2355: self.u = 0
        if self.u > -1.1778 and self.u < -0.7065: self.u = -0.942
        if self.u > 1.64925 and self.u < -1.64925: self.u = 1.885
        if self.u > 0.7065 and self.u < 1.1778: self.u = 0.942
        if self.u > 0.2355 and self.u < 0.7065: self.u = 0.471
        if self.u > 1.1778 and self.u < 1.64925: self.u = 1.4135
        if self.u > -1.64925 and self.u < -1.1778: self.u = -1.4135
        if self.u > -0.7065 and self.u < -0.2355: self.u = -0.471
        self.psi_dot = self.u/v
        self.psi = self.psi_dot * self.t
        self.x_dot = v*(np.cos(self.psi))
        self.y_dot = v*(np.sin(self.psi))
        
        new_x = self.x_dot * self.t
        new_y = self.y_dot * self.t
        
        positionx += new_x
        positiony += new_y
        
        if positionx < self.min_positionx: positionx = self.min_positionx
        if positionx > self.max_positionx: positionx = self.max_positionx
        
        if positiony < self.min_positiony: positiony = self.min_positiony
        if positiony > self.max_positiony: positiony = self.max_positiony
        
        
        self.angle -= 1
        
        
        #------------=========RCS==========----------------
        self.g = 9.8    #9.8 m/s
        self.z = 800000  #800km altitude
        self.theta = math.atan(positiony/positionx)           #Azimuth Angle
        self.lamda = self.theta - self.psi + math.pi    #Aspect Angle
        self.phi = math.atan(self.z/((((positionx)**2) + (positiony)**2)**0.5))  #Elevation Angle
        self.mu = math.atan(self.u/self.g)              #Bank Angle
        
        
        #self.rcsx = math.acos(math.cos(self.phi) * math.cos(self.lamda))
        #self.rcsy = self.mu - (math.atan(math.tan(self.phi)/math.sin(self.lamda)))
        self.a = 0.3172
        self.b = 0.1784
        self.c = 1.003
        self.lamda_e = math.acos(math.cos(self.phi) * math.cos(self.lamda))
        self.mu_e = self.mu -(math.atan(math.tan(self.phi)/math.sin(self.lamda)))
        self.rcs = (math.pi * self.a**2 *self.b**2 * self.c**2)/((self.a**2 * math.sin(self.lamda_e)**2 * math.cos(self.mu_e)**2) + (self.b**2 * math.sin(self.lamda_e)**2 * math.sin(self.mu_e)**2) + (self.c**2 * math.cos(self.lamda_e)**2))
        rcs1 = rcs2 = rcs3 = rcs4 = rcs5 = rcs6 = self.rcs

        #---------========Signal to noise Ratio=======--------------
        self.P_avg = 0.5*((10)**6)  #Average Power = 0.5 Mega Watt 
        self.A_e = 4.9*2.7        #Antenna Area = 4.9m width * 2.7m Height
        self.t_s = 30             #Scan time for solid angle searched = 30sec
        self.omega = math.radians(360) #360 degree solid angle searched
        #-----------===========For Radar 1============----------------
        if positionx < self.radxaxis1 and positiony == self.radyaxis1:            
            self.R1 = (self.radxaxis1 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis1 and positionx == self.radxaxis1:            
            self.R1 = (self.radyaxis1 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis1 and positiony == self.radyaxis1:            
            self.R1 = (positionx - self.radxaxis1 ) * 1000             #Range = 20km
        if positiony > self.radyaxis1 and positionx == self.radxaxis1:            
            self.R1 = (positiony - self.radyaxis1 ) * 1000             #Range = 20km
            
        if positionx < self.radxaxis1 and positiony < self.radyaxis1:            
            self.R1 = (((self.radxaxis1 - positionx) * 1000) + ((self.radyaxis1 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis1 and positionx > self.radxaxis1:            
            self.R1 = (((self.radyaxis1 - positiony) * 1000) + ((positionx - self.radxaxis1) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis1 and positiony > self.radyaxis1:            
            self.R1 = (((positiony - self.radyaxis1) * 1000) + ((self.radxaxis1 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis1 and positionx > self.radxaxis1:            
            self.R1 = (((positiony - self.radyaxis1) * 1000) + ((positionx - self.radxaxis1) * 1000)) * 0.5             #Range = 20km

        #----------===============For Radar 2================------------------
        if positionx < self.radxaxis2 and positiony == self.radyaxis2:            
            self.R2 = (self.radxaxis2 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis2 and positionx == self.radxaxis2:            
            self.R2 = (self.radyaxis2 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis2 and positiony == self.radyaxis2:            
            self.R2 = (positionx - self.radxaxis2 ) * 1000             #Range = 20km
        if positiony > self.radyaxis2 and positionx == self.radxaxis2:            
            self.R2 = (positiony - self.radyaxis2 ) * 1000             #Range = 20km
            
        if positionx < self.radxaxis2 and positiony < self.radyaxis2:            
            self.R2 = (((self.radxaxis2 - positionx) * 1000) + ((self.radyaxis2 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis2 and positionx > self.radxaxis2:            
            self.R2 = (((self.radyaxis2 - positiony) * 1000) + ((positionx - self.radxaxis2) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis2 and positiony > self.radyaxis2:            
            self.R2 = (((positiony - self.radyaxis2) * 1000) + ((self.radxaxis2 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis2 and positionx > self.radxaxis2:            
            self.R2 = (((positiony - self.radyaxis2) * 1000) + ((positionx - self.radxaxis2) * 1000)) * 0.5             #Range = 20km
            
            
        #----------===============For Radar 3================------------------
        if positionx < self.radxaxis3 and positiony == self.radyaxis3:            
            self.R3 = (self.radxaxis3 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis3 and positionx == self.radxaxis3:            
            self.R3 = (self.radyaxis3 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis3 and positiony == self.radyaxis3:            
            self.R3 = (positionx - self.radxaxis3 ) * 1000             #Range = 20km
        if positiony > self.radyaxis3 and positionx == self.radxaxis3:            
            self.R3 = (positiony - self.radyaxis3 ) * 1000             #Range = 20km
            
        if positionx < self.radxaxis3 and positiony < self.radyaxis3:            
            self.R3 = (((self.radxaxis3 - positionx) * 1000) + ((self.radyaxis3 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis3 and positionx > self.radxaxis3:            
            self.R3 = (((self.radyaxis3 - positiony) * 1000) + ((positionx - self.radxaxis3) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis3 and positiony > self.radyaxis3:            
            self.R3 = (((positiony - self.radyaxis3) * 1000) + ((self.radxaxis3 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis3 and positionx > self.radxaxis3:            
            self.R3 = (((positiony - self.radyaxis3) * 1000) + ((positionx - self.radxaxis3) * 1000)) * 0.5             #Range = 20km
            
            
        #----------===============For Radar 4================------------------
        if positionx < self.radxaxis4 and positiony == self.radyaxis4:            
            self.R4 = (self.radxaxis4 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis4 and positionx == self.radxaxis4:            
            self.R4 = (self.radyaxis4 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis4 and positiony == self.radyaxis4:            
            self.R4 = (positionx - self.radxaxis4 ) * 1000             #Range = 20km
        if positiony > self.radyaxis4 and positionx == self.radxaxis4:            
            self.R4 = (positiony - self.radyaxis4 ) * 1000             #Range = 20km
            
        if positionx < self.radxaxis4 and positiony < self.radyaxis4:            
            self.R4 = (((self.radxaxis4 - positionx) * 1000) + ((self.radyaxis4 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis4 and positionx > self.radxaxis4:            
            self.R4 = (((self.radyaxis4 - positiony) * 1000) + ((positionx - self.radxaxis4) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis4 and positiony > self.radyaxis4:            
            self.R4 = (((positiony - self.radyaxis4) * 1000) + ((self.radxaxis4 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis4 and positionx > self.radxaxis4:            
            self.R4 = (((positiony - self.radyaxis4) * 1000) + ((positionx - self.radxaxis4) * 1000)) * 0.5             #Range = 20km
        
        
        #----------===============For Radar 5================------------------
        if positionx < self.radxaxis5 and positiony == self.radyaxis5:            
            self.R5 = (self.radxaxis5 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis5 and positionx == self.radxaxis5:            
            self.R5 = (self.radyaxis5 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis5 and positiony == self.radyaxis5:            
            self.R5 = (positionx - self.radxaxis5 ) * 1000             #Range = 20km
        if positiony > self.radyaxis5 and positionx == self.radxaxis5:            
            self.R5 = (positiony - self.radyaxis5 ) * 1000             #Range = 20km
            
        if positionx < self.radxaxis5 and positiony < self.radyaxis5:            
            self.R5 = (((self.radxaxis5 - positionx) * 1000) + ((self.radyaxis5 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis5 and positionx > self.radxaxis5:            
            self.R5 = (((self.radyaxis5 - positiony) * 1000) + ((positionx - self.radxaxis5) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis5 and positiony > self.radyaxis5:            
            self.R5 = (((positiony - self.radyaxis5) * 1000) + ((self.radxaxis5 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis5 and positionx > self.radxaxis5:            
            self.R5 = (((positiony - self.radyaxis5) * 1000) + ((positionx - self.radxaxis5) * 1000)) * 0.5             #Range = 20km
        
        
        #----------===============For Radar 6================------------------
        if positionx < self.radxaxis6 and positiony == self.radyaxis6:            
            self.R6 = (self.radxaxis6 - positionx) * 1000             #Range = 20km
        if positiony < self.radyaxis6 and positionx == self.radxaxis6:            
            self.R6 = (self.radyaxis6 - positiony) * 1000             #Range = 20km   
        if positionx > self.radxaxis6 and positiony == self.radyaxis6:            
            self.R6 = (positionx - self.radxaxis6) * 1000             #Range = 20km
        if positiony > self.radyaxis6 and positionx == self.radxaxis6:            
            self.R6 = (positiony - self.radyaxis6) * 1000             #Range = 20km
            
        if positionx < self.radxaxis6 and positiony < self.radyaxis6:            
            self.R6 = (((self.radxaxis6 - positionx) * 1000) + ((self.radyaxis6 - positiony) * 1000)) * 0.5             #Range = 20km
        if positiony < self.radyaxis6 and positionx > self.radxaxis6:            
            self.R6 = (((self.radyaxis6 - positiony) * 1000) + ((positionx - self.radxaxis6) * 1000)) * 0.5             #Range = 20km
        if positionx < self.radxaxis6 and positiony > self.radyaxis6:            
            self.R6 = (((positiony - self.radyaxis6) * 1000) + ((self.radxaxis6 - positionx) * 1000)) * 0.5             #Range = 20km
        if positiony > self.radyaxis6 and positionx > self.radxaxis6:            
            self.R6 = (((positiony - self.radyaxis6) * 1000) + ((positionx - self.radxaxis6) * 1000)) * 0.5             #Range = 20km
        
        
#        self.R = 100 * 1000
        self.K = 1.38*((10)**-23)      #Boltzman Constant
        self.T_s = 500              #System noise Temp = 500 deg Celsius
        self.L = 10                 #Total System Loss = 10dB (decibel)
        
        #SNR for Radar 1
        if positionx > self.minradx1 and positiony > self.minrady1 and positionx < self.maxradx1 and positiony < self.maxrady1:
            self.snr1 = (self.P_avg * self.A_e * self.t_s * rcs1)/(4 * math.pi * self.omega * ((self.R1)**4) * self.K * self.T_s * self.L)
        else:
            self.snr1 = 0   
        snr1 = self.snr1
        
        
        #SNR for Radar 2
        if positionx > self.minradx2 and positiony > self.minrady2 and positionx < self.maxradx2 and positiony < self.maxrady2:
            self.snr2 = (self.P_avg * self.A_e * self.t_s * rcs2)/(4 * math.pi * self.omega * ((self.R2)**4) * self.K * self.T_s * self.L)

        else:
            self.snr2 = 0   
        snr2 = self.snr2
        
        
        #SNR for Radar 3
        if positionx > self.minradx3 and positiony > self.minrady3 and positionx < self.maxradx3 and positiony < self.maxrady3:
            self.snr3 = (self.P_avg * self.A_e * self.t_s * rcs3)/(4 * math.pi * self.omega * ((self.R3)**4) * self.K * self.T_s * self.L)

        else:
            self.snr3 = 0   
        snr3 = self.snr3
        
        
        #SNR for Radar 4
        if positionx > self.minradx4 and positiony > self.minrady4 and positionx < self.maxradx4 and positiony < self.maxrady4:
            self.snr4 = (self.P_avg * self.A_e * self.t_s * rcs4)/(4 * math.pi * self.omega * ((self.R4)**4) * self.K * self.T_s * self.L)

        else:
            self.snr4 = 0   
        snr4 = self.snr4
        
        
        #SNR for Radar 5
        if positionx > self.minradx5 and positiony > self.minrady5 and positionx < self.maxradx5 and positiony < self.maxrady5:
            self.snr5 = (self.P_avg * self.A_e * self.t_s * rcs5)/(4 * math.pi * self.omega * ((self.R5)**4) * self.K * self.T_s * self.L)

        else:
            self.snr5 = 0   
        snr5 = self.snr5
        
        
        #SNR for Radar 6
        if positionx > self.minradx6 and positiony > self.minrady6 and positionx < self.maxradx6 and positiony < self.maxrady6:
            self.snr6 = (self.P_avg * self.A_e * self.t_s * rcs6)/(4 * math.pi * self.omega * ((self.R6)**4) * self.K * self.T_s * self.L)

        else:
            self.snr6 = 0   
        snr6 = self.snr6

        #-------===========Conditions========--------
        
#        if (position[0] > self.max_position[0] & position[1] > self.max_position[1]): position = self.max_position
#        if (position[0] < self.min_position[0] & position[1] < self.min_position[1]): position = self.min_position

        done = bool(positionx == self.target_positionx and positiony == self.target_positiony)
        tracked1 = bool(snr1 > 50000000)
        tracked2 = bool(snr2 > 50000000)
        tracked3 = bool(snr3 > 50000000)
        tracked4 = bool(snr4 > 50000000)
        tracked5 = bool(snr5 > 50000000)
        tracked6 = bool(snr6 > 50000000)
#        tracked = bool(snr1 > 50000000 or snr2 > 50000000)

        reward = 0
        if positionx < self.target_positionx: reward-= math.pow(action[0],2)*0.1
        if positiony < self.target_positiony: reward-= math.pow(action[0],2)*0.1
        if positionx > self.target_positionx: reward-= math.pow(action[0],2)*0.1
        if positiony > self.target_positiony: reward-= math.pow(action[0],2)*0.1
        
#        if positionx < self.min_positionx: reward-= math.pow(action[0],2)*0.1
#        if positiony < self.min_positiony: reward-= math.pow(action[0],2)*0.1
#        if positionx == self.target_positionx and positiony != self.target_positiony: reward-= math.pow(action[0],2)*0.1
#        if positiony == self.target_positiony and positionx != self.target_positionx: reward-= math.pow(action[0],2)*0.1
#        if positionx > self.state[0] and positiony > self.state[1]: reward+= (math.pow(action[0],2)*0.1)
#        
#        if positionx > self.state[0]: reward+= math.pow(action[0],2)*0.1
#        if positiony > self.state[1]: reward+= math.pow(action[0],2)*0.1
#        if positionx > self.state[0] and positiony == self.state[1]: reward-= 2*(math.pow(action[0],2)*0.1)
#        if positiony > self.state[1] and positionx == self.state[0]: reward-= 2*(math.pow(action[0],2)*0.1)
        
        
        
        
#        if snr > 100000 and snr < 50000000: reward-= 2 * (math.pow(action[0],2)*0.1)
#        if tracked: reward-= 20 * (math.pow(action[0],2)*0.1)
#        if tracked1: reward-= 20 * (math.pow(action[0],2)*0.1)
#        if tracked2: reward-= 20 * (math.pow(action[0],2)*0.1)
        if positionx == positiony and positionx < self.minradx1: reward += 0.3
        if positionx == positiony and positionx > self.maxradx1: reward += 0.3
#        if positiony > self.maxrady1 and positiony < self.minrady2 and positionx > self.minradx1 and positionx > self.maxradx2: reward += 0.3
        if positiony < self.minrady1 and positionx > self.minradx1: reward -=0.3
        if positionx == self.target_positionx and positiony < self.target_positiony: reward -=0.2
        if positionx > self.minradx1 and positiony < self.radyaxis1: reward -=0.3
        
        
#        if snr1 > 1 and snr1 < 10000000 and snr2 > 1 and snr2 < 10000000:  reward+= 0.5
#        if snr1 > 10000000 and snr1 < 20000000 and snr2 > 10000000 and snr2 < 20000000:  reward+= 0.2
#        
#        if snr2 > 1 and snr2 < 10000000 and snr3 > 1 and snr3 < 10000000:  reward+= 0.5
#        if snr2 > 10000000 and snr2 < 20000000 and snr3 > 10000000 and snr3 < 20000000:  reward+= 0.2
        
        if snr3 > 1 and snr3 < 10000000 and snr4 > 1 and snr4 < 10000000:  reward+= 0.5
        if snr3 > 10000000 and snr3 < 15000000 and snr4 > 10000000 and snr4 < 15000000:  reward+= 0.2
        
#        if snr4 > 1 and snr4 < 10000000 and snr5 > 1 and snr5 < 10000000:  reward+= 0.5
#        if snr4 > 10000000 and snr4 < 20000000 and snr5 > 10000000 and snr5 < 20000000:  reward+= 0.2
        
#        if snr5 > 1 and snr5 < 10000000 and snr6 > 1 and snr6 < 10000000:  reward+= 0.5
#        if snr5 > 10000000 and snr5 < 20000000 and snr6 > 10000000 and snr6 < 20000000:  reward+= 0.2
        
#        if snr1 > 10000000 and snr1 < 20000000:  reward+= 0.1
        if snr1 > 20000000 and snr1 < 30000000:  reward-= 0.1
        if snr1 > 30000000 and snr1 < 40000000:  reward-= 0.2
        if snr1 > 40000000 and snr1 < 50000000:  reward-= 0.3
        
#        if snr2 > 10000000 and snr2 < 20000000:  reward+= 0.1
        if snr2 > 20000000 and snr2 < 30000000:  reward-= 0.1
        if snr2 > 30000000 and snr2 < 40000000:  reward-= 0.2
        if snr2 > 40000000 and snr2 < 50000000:  reward-= 0.3
        
#        if snr3 > 10000000 and snr3 < 20000000:  reward+= 0.1
        if snr3 > 20000000 and snr3 < 30000000:  reward-= 0.1
        if snr3 > 30000000 and snr3 < 40000000:  reward-= 0.2
        if snr3 > 40000000 and snr3 < 50000000:  reward-= 0.3
        
#        if snr4 > 10000000 and snr4 < 20000000:  reward+= 0.1
        if snr4 > 20000000 and snr4 < 30000000:  reward-= 0.1
        if snr4 > 30000000 and snr4 < 40000000:  reward-= 0.2
        if snr4 > 40000000 and snr4 < 50000000:  reward-= 0.3
        
#        if snr5 > 10000000 and snr5 < 20000000:  reward+= 0.1
        if snr5 > 20000000 and snr5 < 30000000:  reward-= 0.1
        if snr5 > 30000000 and snr5 < 40000000:  reward-= 0.2
        if snr5 > 40000000 and snr5 < 50000000:  reward-= 0.3
        
#        if snr6 > 10000000 and snr6 < 20000000:  reward+= 0.1
        if snr6 > 20000000 and snr6 < 30000000:  reward-= 0.1
        if snr6 > 30000000 and snr6 < 40000000:  reward-= 0.2
        if snr6 > 40000000 and snr6 < 50000000:  reward-= 0.3
        if tracked1: reward-= 2         #2
        if tracked2: reward-= 2
        if tracked3: reward-= 2
        if tracked4: reward-= 2
        if tracked5: reward-= 2 
        if tracked6: reward-= 2

        if done:
            reward += 100.0
#        else: 
#        reward-= math.pow(action[0],2)*0.1
#        else: reward -= 1

        self.state = np.array([positionx, positiony, rcs1, snr1, rcs2, snr2, rcs3, snr3, rcs4, snr4, rcs5, snr5, rcs6, snr6])
        return self.state, reward, done, {}

    def reset(self):
        self.state = np.array([stxaxis, styaxis, self.minrcs1, self.minsnr1, self.minrcs2, self.minsnr2, self.minrcs3, self.minsnr3, self.minrcs4, self.minsnr4, self.minrcs5, self.minsnr5, self.minrcs6, self.minsnr6])
        return np.array(self.state)

#    def get_state(self):
#        return self.state

#    def _height(self, xs):
#        return np.sin(3 * xs)*.45+.55

    def render(self, mode='human'):
        screen_width = WIDTH
        screen_height = HEIGHT

        if self.viewer is None:
            from gym.envs.classic_control import rendering
            self.viewer = rendering.Viewer(screen_width, screen_height)
            l = 0
            r = screen_width
            t = screen_height
            b = 0
            ar = rendering.FilledPolygon([(l,b), (l,t), (r,t), (r,b)])
            self.viewer.add_geom(ar)
            
            
            
#            Radar
#            for count in range(1, radar_count+1, 1):
#                radlin1 = self.viewer.draw_polyline([(self.radxaxis-(self.radrange-20),self.radyaxis),(self.radxaxis+(self.radrange-20),self.radyaxis)], color=(0,255,0), linewidth=1)
#                radlin2 = self.viewer.draw_polyline([(self.radxaxis,self.radyaxis-(self.radrange-20)),(self.radxaxis,self.radyaxis+(self.radrange-20))], color=(0,255,0), linewidth=1)
#                self.viewer.add_geom(radlin1)
#                self.viewer.add_geom(radlin2)
            for count in range(1, radar_count+1, 1):
                for x in range(1,radrange[count - 1]+10,10):    
                    rad = rendering.make_circle(radius=x, filled=False)
                    rad.set_color(0, 255, 0)
                    self.cat = rendering.Transform()
                    rad.add_attr(self.cat)
                    self.viewer.add_geom(rad)
                    self.cat.set_translation(radxaxis[count - 1],radyaxis[count - 1])



            #Main Length
            l = 0
            r = 20
            t = 13
            b = 7
            air = rendering.FilledPolygon([(l,b), (l,t), (r,t), (r,b)])
            air.set_color(255, 0, 0)
            self.viewer.add_geom(air)
            self.airtrans = rendering.Transform()
            air.add_attr(self.airtrans)
            
            #Front left
            flagx = 0
            flagy2 = 0
            airf = rendering.FilledPolygon([(flagx, flagy2), (flagx+7, flagy2), (flagx, flagy2+15)])
            airf.set_color(255, 0, 0)
            self.cartt = rendering.Transform()
            airf.add_attr(self.cartt)
            self.viewer.add_geom(airf)
            self.cartt.set_translation(9,13)
            airf.add_attr(self.airtrans)
            
            #Front Right
            flagx = -7
            flagy2 = 0
            airff = rendering.FilledPolygon([(flagx, flagy2), (flagx+7, flagy2), (flagx+7, flagy2+15)])
            airff.set_color(255, 0, 0)
            self.carttt = rendering.Transform()
            airff.add_attr(self.carttt)
            self.viewer.add_geom(airff)
            self.carttt.set_translation(9,7)
            self.carttt.set_rotation(math.radians(180))
            airff.add_attr(self.airtrans)
            
            #Back left
            flagx = 0
            flagy2 = 0
            airb = rendering.FilledPolygon([(flagx, flagy2), (flagx+4, flagy2), (flagx, flagy2+8)])
            airb.set_color(255, 0, 0)
            self.carttb = rendering.Transform()
            airb.add_attr(self.carttb)
            self.viewer.add_geom(airb)
            self.carttb.set_translation(0,13)
            airb.add_attr(self.airtrans)
            
            #Back Right
            flagx = -4
            flagy2 = 0
            airbb = rendering.FilledPolygon([(flagx, flagy2), (flagx+4, flagy2), (flagx+4, flagy2+8)])
            airbb.set_color(255, 0, 0)
            self.cartttb = rendering.Transform()
            airbb.add_attr(self.cartttb)
            self.viewer.add_geom(airbb)
            self.cartttb.set_translation(0,7)
            self.cartttb.set_rotation(math.radians(180))
            airbb.add_attr(self.airtrans)

            #Cone
            airc = rendering.make_circle(2.8)
            airc.set_color(255, 0, 0)
            self.cart = rendering.Transform()
            airc.add_attr(self.cart)
            self.viewer.add_geom(airc)
            self.cart.set_translation(19, 10)
            airc.add_attr(self.airtrans)
            
            #self.airtrans.set_translation(300,300)
            #self.airtrans.set_rotation(math.radians(270))
            
            
            #Start Position Pole
            lagpole = rendering.Line((stxaxis, styaxis+10), (stxaxis, styaxis-30))
            lagpole.set_color(0,255,0)
            self.viewer.add_geom(lagpole)
            lag = rendering.FilledPolygon([(stxaxis, styaxis+10), (stxaxis, styaxis), (stxaxis+25, styaxis+10-5)])
            lag.set_color(32,178,170)
            self.viewer.add_geom(lag)
            
            #Target Pole
            flagpole = rendering.Line((endxaxis, endyaxis+10), (endxaxis, endyaxis-30))
            flagpole.set_color(255,0,0)
            self.viewer.add_geom(flagpole)
            flag = rendering.FilledPolygon([(endxaxis, endyaxis+10), (endxaxis, endyaxis), (endxaxis+25, endyaxis+10-5)])
            flag.set_color(.8,.8,0)
            self.viewer.add_geom(flag)
            
            #For radar 1
            movlin1 = rendering.Line((0, 0), (self.radrange1, 0))
            movlin1.set_color(255,0,0)
            self.catt = rendering.Transform()
            movlin1.add_attr(self.catt)
            self.catt.set_translation(self.radxaxis1,self.radyaxis1)
            self.viewer.add_geom(movlin1)
            
            movlin2 = rendering.Line((0, 0), (self.radrange1, 0))
            movlin2.set_color(255,0,0)
            self.catt2 = rendering.Transform()
            movlin2.add_attr(self.catt2)
            self.catt2.set_translation(self.radxaxis1,self.radyaxis1)
            self.catt2.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin2)
            
            
            #For Radar 2
            movlin11 = rendering.Line((0, 0), (self.radrange2, 0))
            movlin11.set_color(255,0,0)
            self.catt1 = rendering.Transform()
            movlin11.add_attr(self.catt1)
            self.catt1.set_translation(self.radxaxis2,self.radyaxis2)
            self.viewer.add_geom(movlin11)
            
            movlin21 = rendering.Line((0, 0), (self.radrange2, 0))
            movlin21.set_color(255,0,0)
            self.catt21 = rendering.Transform()
            movlin21.add_attr(self.catt21)
            self.catt21.set_translation(self.radxaxis2,self.radyaxis2)
            self.catt21.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin21)
            
            
            #For radar 3
            movlin13 = rendering.Line((0, 0), (self.radrange3, 0))
            movlin13.set_color(255,0,0)
            self.catt3 = rendering.Transform()
            movlin13.add_attr(self.catt3)
            self.catt3.set_translation(self.radxaxis3,self.radyaxis3)
            self.viewer.add_geom(movlin13)
            
            movlin23 = rendering.Line((0, 0), (self.radrange3, 0))
            movlin23.set_color(255,0,0)
            self.catt23 = rendering.Transform()
            movlin23.add_attr(self.catt23)
            self.catt23.set_translation(self.radxaxis3,self.radyaxis3)
            self.catt23.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin23)
            
            
            #For Radar 4
            movlin114 = rendering.Line((0, 0), (self.radrange4, 0))
            movlin114.set_color(255,0,0)
            self.catt14 = rendering.Transform()
            movlin114.add_attr(self.catt14)
            self.catt14.set_translation(self.radxaxis4,self.radyaxis4)
            self.viewer.add_geom(movlin114)
            
            movlin214 = rendering.Line((0, 0), (self.radrange4, 0))
            movlin214.set_color(255,0,0)
            self.catt214 = rendering.Transform()
            movlin214.add_attr(self.catt214)
            self.catt214.set_translation(self.radxaxis4,self.radyaxis4)
            self.catt214.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin214)
            
            
            #For radar 5
            movlin15 = rendering.Line((0, 0), (self.radrange5, 0))
            movlin15.set_color(255,0,0)
            self.catt5 = rendering.Transform()
            movlin15.add_attr(self.catt5)
            self.catt5.set_translation(self.radxaxis5,self.radyaxis5)
            self.viewer.add_geom(movlin15)
            
            movlin25 = rendering.Line((0, 0), (self.radrange5, 0))
            movlin25.set_color(255,0,0)
            self.catt25 = rendering.Transform()
            movlin25.add_attr(self.catt25)
            self.catt25.set_translation(self.radxaxis5,self.radyaxis5)
            self.catt25.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin25)
            
            
            #For Radar 6
            movlin116 = rendering.Line((0, 0), (self.radrange6, 0))
            movlin116.set_color(255,0,0)
            self.catt16 = rendering.Transform()
            movlin116.add_attr(self.catt16)
            self.catt16.set_translation(self.radxaxis6,self.radyaxis6)
            self.viewer.add_geom(movlin116)
            
            movlin216 = rendering.Line((0, 0), (self.radrange6, 0))
            movlin216.set_color(255,0,0)
            self.catt216 = rendering.Transform()
            movlin216.add_attr(self.catt216)
            self.catt216.set_translation(self.radxaxis6,self.radyaxis6)
            self.catt216.set_rotation(math.radians(360 - 60))
            self.viewer.add_geom(movlin216)
            


        positionx = self.state[0]
        positiony = self.state[1]
        self.airtrans.set_translation(round(positionx), round(positiony))
        #print((pos-self.min_position)*scale, self._height(pos)*scale)
#        self.u = self.action[0]

#        self.psi_dot = self.u/v
#        self.psi = self.psi_dot * self.t
#        self.airtrans.set_rotation(self.psi)
#       # print(math.cos(3 * pos))
        
        self.catt.set_rotation(math.radians(self.angle))
        self.catt2.set_rotation(math.radians(self.angle - 60))
        
        self.catt1.set_rotation(math.radians(self.angle))
        self.catt21.set_rotation(math.radians(self.angle - 60))
        
        self.catt3.set_rotation(math.radians(self.angle))
        self.catt23.set_rotation(math.radians(self.angle - 60))
        
        self.catt14.set_rotation(math.radians(self.angle))
        self.catt214.set_rotation(math.radians(self.angle - 60))
        
        self.catt5.set_rotation(math.radians(self.angle))
        self.catt25.set_rotation(math.radians(self.angle - 60))
        
        self.catt16.set_rotation(math.radians(self.angle))
        self.catt216.set_rotation(math.radians(self.angle - 60))

        return self.viewer.render(return_rgb_array = mode=='rgb_array')

    def close(self):
        if self.viewer:
            self.viewer.close()
            self.viewer = None